@@ -71,7 +71,6 @@ FfmpegOpenInput(AVIOContext *pb,
 
 	int err = avformat_open_input(&context, filename, fmt, nullptr);
 	if (err < 0) {
-		avformat_free_context(context);
 		SetFfmpegError(error, err, "avformat_open_input() failed");
 		return nullptr;
 	}
@@ -643,7 +642,7 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 			/* end of file */
 			break;
 
-		if (packet.stream_index == audio_stream) {
+		if (packet.size > 0 && packet.stream_index == audio_stream) {
 			cmd = ffmpeg_send_packet(decoder, input,
 						 packet,
 						 *codec_context,
