@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,7 @@
 #include <pthread.h>
 #endif
 
+#include <assert.h>
 #include <signal.h>
 
 class SignalMonitor final : private SocketMonitor {
@@ -61,14 +62,6 @@ public:
 #endif
 	}
 
-	~SignalMonitor() {
-		/* prevent the descriptor to be closed twice */
-#ifdef USE_SIGNALFD
-		if (SocketMonitor::IsDefined())
-#endif
-			SocketMonitor::Steal();
-	}
-
 	using SocketMonitor::GetEventLoop;
 
 #ifdef USE_SIGNALFD
